#!/bin/sh

. /usr/ramdisk/PG.conf
. /usr/ramdisk/etc/panabit.inf

USER_DIR="/usr/ramdisk/etc/webuser"
WEBLOGIN="/usr/ramdisk/tmp/weblogin"
ESCTOOL="/usr/ramdisk/bin/esctool"
FLOWEYE="/usr/ramdisk/bin/floweye"

PALANG="ch"

PALANGCONF="${PGETC}/palang.conf"
SYSNAMECONF="${PGETC}/sysname.conf"
WEBACL="${PGETC}/App/systemgd/webacl.conf"
Auth_type_file="${PGETC}/App/opser/auth_type.conf"


[ -f ${PALANGCONF} ] && . ${PALANGCONF}
[ -f ${Auth_type_file} ] && . ${Auth_type_file}
[ -f ${PGETC}/ifadmin.conf ] && . ${PGETC}/ifadmin.conf
[ -f ${WEBACL} ] && . ${WEBACL}


retjson()
{
	printf "Content-Type:application/json;charset=gb2312\r\n"
	printf "\r\n"
	
	if [ "${3}" = "" ]; then
		echo "{ \"code\": ${1}, \"msg\": \"${2}\", \"data\": \"\"}"
	else
		echo "{ \"code\": ${1}, \"msg\": \"${2}\", \"data\": ${3}}"
	fi

	exit 0
}


WEB_LOGGER()
{	
	# $1 User
	# $2 Action
	# $3 Message

	local logfile="${DATAPATH}/web_`date +%Y.%m.%d`.log"
	local curtime=`date +%Y.%m.%d/%H:%M:%S`

	echo "${curtime} ${REMOTE_ADDR} $1 $2 $3" >> ${logfile}
	sync
	sync
}


load_login_env()
{
	SYSNAME=""

	[ -f ${SYSNAMECONF} ] && . ${SYSNAMECONF}

	[ "${ENONLY}" = "" ] && ENONLY=0
	[ "${NTM}" = "1" ] && CHONLY=1
	blockTime=`verify_errnum get`
	
	printf "{"
	printf "\"lang\":\"${PALANG}\","
	printf "\"enonly\":\"${ENONLY}\","
	printf "\"chonly\":\"${CHONLY}\","
	printf "\"title\":\"${SYSNAME}\","
	printf "\"verifyCode\":\"${verifyCode}\","
	printf "\"maxErr\":\"${maxErr}\","
	printf "\"lockTime\":\"${lockTime}\","
	printf "\"blockTime\":\"${blockTime}\","
	printf "\"pswdTTL\":\"${pswdTTL}\""
	printf "}"
}


record_errnum()
{
	[ "${maxErr}" = "0" ] && return

	mkdir -p ${WEBLOGIN}
	error_file="${WEBLOGIN}/FAIL_${REMOTE_ADDR}"
	ERRNUM=1

	if [ -f ${error_file} ]; then
		. ${error_file}
		ERRNUM=$((${ERRNUM}+1))
	fi

	echo "ERRNUM=${ERRNUM}" > ${error_file}
	echo "LEFT=`date +%s`" >> ${error_file}

	if [ ${ERRNUM} -eq ${maxErr} ]; then
		LOG_MSG="¼¼"
		[ "${PALANG}" = "en" ] && LOG_MSG="LOGIN_BLOCK"
		WEB_LOGGER "${src_username}" "${LOG_MSG}" "user=${src_username},ip=${REMOTE_ADDR}"
		retjson 3 "ﵽ¼ҳ"
	fi
}


verify_errnum()
{
	error_file="${WEBLOGIN}/FAIL_${REMOTE_ADDR}"
	
	if [ "${maxErr}" = "0" ]; then
		[ -f "${error_file}" ] && rm ${error_file}
		return
	fi

	[ ! -f "${error_file}" ] && return

	. ${error_file}
	
	[ ${ERRNUM} -lt ${maxErr} ] && return

	now=`date +%s`
	free=$((${now}-${LEFT}))
	if [ ${free} -lt ${lockTime} ]; then
		if [ "${1}" = "get" ]; then
			echo $((${lockTime}-${free}))
		else
			retjson 1 "LOGIN_BLOCK" $((${lockTime}-${free}))
		fi
	else
		rm ${error_file}
	fi
}


cookie_make()
{
	logintime=`date +%s1`
	code=`uuidgen | cut -c5-8`
	esc_code=`${ESCTOOL} -e ${code}`
	username=`${ESCTOOL} -d ${CGI_username}`
	echo "paonline_${username}_$$_${logintime}_${esc_code}|${SERVER_PORT}|"
}


code_make()
{
	mkdir -p ${WEBLOGIN}
	code=`uuidgen | cut -c5-8`
	cfile="${WEBLOGIN}/CODE_${REMOTE_ADDR}"
	echo "CODE=${code}" > ${cfile}
	echo "LIVE=`date +%s`" >> ${cfile}
}


code_post()
{
	cfile="${WEBLOGIN}/CODE_${REMOTE_ADDR}"
	
	[ ! -f ${cfile} ] && code_make

	. ${cfile}
	
	CODE=`${ESCTOOL} -e ${CODE}`
	retjson 0 "OK" "\"${CODE}\""
}


code_cmp()
{
	cfile="${WEBLOGIN}/CODE_${REMOTE_ADDR}"

	[ ! -f ${cfile} ] && return
	
	if [ "${CGI_code}" = "" ]; then
		code_make
		retjson 1 "֤"
	fi

	. ${cfile}
	
	now=`date +%s`
	
	if [ $((${now} - ${LIVE})) -gt 60 ]; then
		code_make
		retjson 1 "֤볬ʱ"
	fi

	if [ "${CGI_code}" != "${CODE}" ]; then
		code_make
		retjson 1 "֤"
	fi

	[ -f ${cfile} ] && rm -f ${cfile}
}


radius_auth()
{
	RADCLIENT=/usr/ramdisk/app/opser/bin/radclient

	if [ ! -x ${RADCLIENT} ]; then
		local_auth
		return
	fi

	if [ "${raas_url}" != "" ]; then
		src_name="${src_name}@${raas_url}"
	fi

	${RADCLIENT} -u ${src_username} -p ${src_password} -r ${rad_ip} -i ${ADMIN_IP} -n ${nas_id} -s ${rad_key} -f ${REMOTE_ADDR} -t ${rad_timeout}
}


local_auth()
{
	ufile="${USER_DIR}/${CGI_username}"
	if [ ! -f ${ufile} ]; then
		echo "û"
		return 1
	fi

	. ${ufile}

	[ "${lastedit}" = "" ] && lastedit=`date +%s`
	[ "${pswdTTL}" = "" ] && pswdTTL=0

	# УIPǷ¼
	if [ "${iptab_id}" != "" ]; then
		ipnum=`${FLOWEYE} table get id=${iptab_id} | wc -l | awk '{print $1}'`
		if [ "${ipnum}" != "0" ]; then
			allow_login=`${FLOWEYE} table get allmatched=${REMOTE_ADDR} | grep "${iptab_id} ${iptab_name}"`
			if [ "${allow_login}" = "" ]; then
				echo "IP_DENY"
				return 1
			fi
		fi
	fi

	if [ "${password}" != "${CGI_password}" ]; then
		echo "¼֤ʧ"
		return 1
	fi

	# Уʱ
	if [ "${pswdTTL}" != "0" ]; then
		_ttl=$((${pswdTTL} - (`date +%s`- ${lastedit})))
		if [  ${_ttl} -le 0 ]; then
			echo "PSWD_TIMEOUT"
			return 1
		else
			pswdLeft=$((${_ttl} / 24 /60 / 60))
		fi
	fi

	if [ "${anylogin}" = "deny" ]; then
		is_login=`ls /usr/ramdisk/weblogin/ | grep "paonline_${name}_"`
		if [ "${is_login}" != "" ]; then
			echo "˺ŵǰѾ¼ҽֹͬʱദ¼"
			return 1
		fi
	fi

	echo "AUTH_OK|${pswdLeft}|${level}"
	return 0
}


user_login()
{
	[ "${CGI_username}" = "" ] && retjson 1 "Чû"
	[ "${CGI_password}" = "" ] && retjson 1 "INV_PASSWORD"
	[ "${CGI_palang}" != "ch" -a "${CGI_palang}" != "en" ] && retjson 1 "INV_LANG"

	export src_username=`${ESCTOOL} -d ${CGI_username}`
	export src_password=`${ESCTOOL} -d ${CGI_password}`

	xss=`echo ${src_username} | grep -E "[<*&/>]"`
	[ "${xss}" != "" ] && retjson 1 "Чû"

	verify_errnum
	code_cmp

	# get auth ret
	case "${auth_type}" in
		"radius")
			auth_ret=`radius_auth`
			;;

		"rad_then_local")
			auth_ret=`radius_auth`
			[ "${auth_ret}" = "TIME_OUT" ] && auth_ret=`local_auth`
			;;

		*)
			auth_ret=`local_auth`
	esac

	login_level=`echo "${auth_ret}" | cut -d"|" -f3`
	pswdLeft=`echo "${auth_ret}" | cut -d"|" -f2`
	auth_ret=`echo "${auth_ret}" | cut -d"|" -f1`

	[ "${pswdLeft}" = "" ] && pswdLeft=0

	# handle auth ret
	case "${auth_ret}" in
		"IP_DENY")
			retjson 4 "IPֹ¼豸"
			;;

		"PSWD_TIMEOUT")
			code_make
			retjson 2 "ʧЧ"
			;;

		"AUTH_OK")
			mkdir -p /usr/ramdisk/weblogin/

			ckinfo=`cookie_make`
			ckfile="/usr/ramdisk/weblogin/${ckinfo}"
			[ "${login_level}" = "" ] && login_level=1

			echo "ip=${REMOTE_ADDR}" > ${ckfile}
			echo "user=${src_username}" >> ${ckfile}
			echo "login_level=${login_level}" >> ${ckfile}
			echo "logintime=`date +%Y.%m.%d/%H:%M:%S`" >> ${ckfile}
			echo "PALANG=${CGI_palang}" > ${PALANGCONF}

			WEB_LOGGER  "${src_username}" "û¼" "user=${src_username},ip=${REMOTE_ADDR}"
			${FLOWEYE} logger alert "${src_username} login success from ${REMOTE_ADDR}"
			[ -f "${error_file}" ] && rm ${error_file}

			printf "Content-Type:application/json;charset=gb2312\r\n"
			printf "Set-Cookie: ${PANABIT_CKNAME}=${ckinfo};Secure=true;Path=/;HTTPOnly\r\n"
			printf "\r\n"

			echo -n "{ \"code\": 0, \"msg\": \"Auth_OK\", \"pwleft\": \"${pswdLeft}\"}"
			exit 0
			;;

		*)
			code_make
			[ "${ERRNUM}" = "" ] && ERRNUM=0
			left_changce=$((${maxErr}-${ERRNUM}))
			record_errnum
			${FLOWEYE} logger alert "${src_username} login FAIL from ${REMOTE_ADDR}"
			retjson 1 "${auth_ret}" ${left_changce}
			;;
	esac
}


edit_password()
{
	[ "${CGI_username}" = "" ] && retjson 1 "NO_USERNAME"

	src_username=`${ESCTOOL} -e ${CGI_username}`
	ufile="${USER_DIR}/${CGI_username}"

	[ ! -f ${ufile} ] && retjson 1 "û"

	. ${ufile}
	[ "${CGI_cur}" != ${password} ] && retjson 1 "ǰ"

	echo "name=${name}" > ${ufile}
	echo "password=${CGI_code}" >> ${ufile}
	echo "lastedit=`date +%s`" >> ${ufile}
	echo "level=${level}" >> ${ufile}
	echo "iptab_id=${iptab_id}" >> ${ufile}
	echo "iptab_name=${iptab_name}" >> ${ufile}
	echo "desc=${desc}" >> ${ufile}

	cp "${ufile}" "${PGPATH}/etc/webuser/"
	sync

	WEB_LOGGER "${src_username}" "༭ϵͳû" "username=${src_username}"
	retjson 0 "޸ĳɹ"
}


# action from login.js
case "${CGI_action}" in
	"load_login_env")
		retjson 0 "OK" "`load_login_env`"
		;;

	"getauthcode")
		code_make
		code_post
		;;

	"user_login")
		user_login
		;;

	"edit_password")
		edit_password
		;;
esac


# Login From Cloud or Get userverify.cgi
printf "Content-type: text/html\r\n";
printf "Cache-Control: no-cache\r\n"

if [ "${CGI_pakey}" != "" -a "${CGI_username}" = "admin" ]; then
	keyexist=`${FLOWEYE} cloud list | grep "${CGI_pakey}"`
	if [ "${keyexist}" = "" ]; then
		printf "\r\n"
		echo "<html xmlns:v=\"urn:schemas-microsoft-com:vml\" xmlns:o=\"urn:schemas-microsoft-com:office:office\">"
		echo "<head>"
		echo "<script language=javascript>"
		echo "alert(\"!\");"
		echo "</script>"
		echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=gb2312\">"
		echo "<META HTTP-EQUIV=REFRESH CONTENT=\"0;URL=/login/login.htm\">"
		echo "</head>"
		echo "</html>"

		exit 0
	else
		mkdir -p /usr/ramdisk/weblogin/
		ckinfo=`cookie_make`
		ckfile="/usr/ramdisk/weblogin/${ckinfo}"
	
		${FLOWEYE} cloud remove key=${CGI_pakey}

		echo "ip=${REMOTE_ADDR}" > ${ckfile}
		echo "user=${CGI_username}" >> ${ckfile}
		echo "login_level=0" >> ${ckfile}
		echo "logintime=`date +%Y.%m.%d/%H:%M:%S`" >> ${ckfile}

		printf "Set-Cookie: ${PANABIT_CKNAME}=${ckinfo};Path=/;HTTPOnly;palang=${PALANG}\r\n"
		printf "\r\n"

		echo "<html xmlns:v=\"urn:schemas-microsoft-com:vml\" xmlns:o=\"urn:schemas-microsoft-com:office:office\">"
		echo "<head>"
		echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=gb2312\">"
		echo "<META HTTP-EQUIV=REFRESH CONTENT=\"0;URL=/\">"
		echo "</head>"
		echo "</html>"

		exit 0
	fi
fi

printf "\r\n"
echo "<html xmlns:v=\"urn:schemas-microsoft-com:vml\" xmlns:o=\"urn:schemas-microsoft-com:office:office\">"
echo "<head>"
echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=gb2312\">"
echo "<META HTTP-EQUIV=REFRESH CONTENT=\"0;URL=/login/login.htm\">"
echo "</head>"
echo "</html>"
